@@ -3,79 +3,119 @@ import string
 import typing
 
 from terminusgps.wialon import constants, flags
+from terminusgps.wialon.items import WialonUnit
+from terminusgps.wialon.items.resource import WialonResource
 from terminusgps.wialon.session import WialonSession
 
 
-def get_units_by_carrier(carrier_name: str, session: WialonSession) -> list[str]:
+def get_carrier_names(session: WialonSession) -> list[str]:
     """
-    Returns a list of all unit ids by telecommunications carrier company name.
+    Returns a list of all telecommunication carrier company names present in Wialon.
+
+    :param session: A valid Wialon API session.
+    :type session: :py:obj:`~terminusgps.wialon.session.WialonSession`
+    :returns: A list of telecommunication carrier company names.
+    :rtype: :py:obj:`list`
+
+    """
+    afname: str = constants.WialonItemProperty.ADMIN_FIELD_NAME
+    afval: str = constants.WialonItemProperty.ADMIN_FIELD_VALUE
+    results: dict = session.wialon_api.core_search_items(
+        **{
+            "spec": {
+                "itemsType": constants.WialonItemsType.UNIT,
+                "propName": ",".join([afname, afval]),
+                "propValueMask": "carrier,*",
+                "sortType": ",".join([afname, afval]),
+            },
+            "force": 0,
+            "flags": flags.DataFlag.UNIT_BASE + flags.DataFlag.UNIT_ADMIN_FIELDS,
+            "from": 0,
+            "to": 0,
+        }
+    )
+    carrier_names: list[str] = [
+        field["v"]
+        for item in results.get("items", [{}])
+        for field in item.get("aflds", {}).values()
+        if field["n"] == "carrier" and field["v"]
+    ]
+    return sorted(list(frozenset(carrier_names)))
+
+
+def get_units_by_carrier(
+    carrier_name: str, session: WialonSession
+) -> list[WialonUnit] | None:
+    """
+    Returns a list of all units by telecommunications carrier company name.
 
     :param carrier_name: A telecommunications carrier company name, e.g. ``"US Cell"`` or ``"Conetixx"``.
     :type carrier_name: :py:obj:`str`
     :param session: A valid Wialon API session.
     :type session: :py:obj:`~terminusgps.wialon.session.WialonSession`
-    :returns: A list of unit ids.
+    :returns: A list of units, if any were found.
     :rtype: :py:obj:`list`
 
     """
-    property: str = constants.WialonItemProperty.ADMIN_FIELD_NAME_VALUE.value
     results = session.wialon_api.core_search_items(
         **{
             "spec": {
                 "itemsType": "avl_unit",
-                "propName": property,
+                "propName": constants.WialonItemProperty.ADMIN_FIELD_NAME_VALUE,
                 "propValueMask": f"carrier:{carrier_name}",
-                "sortType": property,
+                "sortType": constants.WialonItemProperty.ADMIN_FIELD_NAME_VALUE,
             },
             "force": 0,
-            "flags": flags.DataFlag.UNIT_BASE.value,
+            "flags": flags.DataFlag.UNIT_BASE,
             "from": 0,
             "to": 0,
         }
     )
-    return [item.get("id") for item in results.get("items", [])]
+    if results:
+        return [
+            WialonUnit(id=item.get("id"), session=session)
+            for item in results.get("items", [])
+        ]
 
 
-def get_unit_by_iccid(iccid: str, session: WialonSession) -> str | None:
+def get_unit_by_iccid(iccid: str, session: WialonSession) -> WialonUnit | None:
     """
-    Returns a unit id by iccid (SIM card #).
+    Returns a unit by iccid (SIM card #).
 
     :param iccid: A SIM card #.
     :type iccid: :py:obj:`str`
     :param session: A valid Wialon API session.
     :type session: :py:obj:`~terminusgps.wialon.session.WialonSession`
-    :returns: The unit id, if it was found.
-    :rtype: :py:obj:`str` | :py:obj:`None`
+    :returns: The unit, if it was found.
+    :rtype: :py:obj:`~terminusgps.wialon.items.WialonUnit` | :py:obj:`None`
 
     """
-    property: str = constants.WialonItemProperty.ADMIN_FIELD_NAME_VALUE.value
     results = session.wialon_api.core_search_items(
         **{
             "spec": {
                 "itemsType": "avl_unit",
-                "propName": property,
+                "propName": constants.WialonItemProperty.ADMIN_FIELD_NAME_VALUE,
                 "propValueMask": f"iccid:{iccid}",
-                "sortType": property,
+                "sortType": constants.WialonItemProperty.ADMIN_FIELD_NAME_VALUE,
             },
             "force": 0,
-            "flags": flags.DataFlag.UNIT_BASE.value,
+            "flags": flags.DataFlag.UNIT_BASE,
             "from": 0,
             "to": 0,
         }
     )
-    if results.get("totalItemsCount") != 1:
-        raise ValueError(results)
-    return results.get("items", [{}])[0].get("id")
+    if results:
+        return WialonUnit(id=results.get("items", [{}])[0].get("id"), session=session)
 
 
-def get_resources(session: WialonSession) -> list[int] | None:
+def get_resources(session: WialonSession) -> list[WialonResource] | None:
     """
-    Returns a list of all resource ids in Wialon.
+    Returns a list of all resources in Wialon.
 
     :param session: A valid Wialon API session.
     :type session: :py:obj:`~terminusgps.wialon.session.WialonSession`
-    :returns: A list of account ids.
-    :rtype: :py:obj:`list`
+    :returns: A list of resources, if found.
+    :rtype: :py:obj:`list` | :py:obj:`None`
 
     """
     results = session.wialon_api.core_search_items(
@@ -95,7 +135,10 @@ def get_resources(session: WialonSession) -> list[int] | None:
         }
     )
     if results:
-        return [int(item.get("id")) for item in results.get("items", [])]
+        return [
+            WialonResource(id=item.get("id"), session=session)
+            for item in results.get("items", [])
+        ]
 
 
 def get_hw_type_id(name: str, session: WialonSession) -> int | None:
@@ -123,7 +166,7 @@ def get_hw_type_id(name: str, session: WialonSession) -> int | None:
     return int(hw_types.get(name)) if name in hw_types.keys() else None
 
 
-def get_unit_by_imei(imei: str, session: WialonSession) -> str | None:
+def get_unit_by_imei(imei: str, session: WialonSession) -> WialonUnit | None:
     """
     Takes a Wialon unit's IMEI # and returns its unit id.
 
@@ -158,7 +201,7 @@ def get_unit_by_imei(imei: str, session: WialonSession) -> str | None:
     )
 
     if results and results.get("totalItemsCount", 0) == 1:
-        return results["items"][0].get("id")
+        return WialonUnit(id=results["items"][0].get("id"), session=session)
 
 
 def get_vin_info(vin_number: str, session: WialonSession) -> dict:
